package com.bizmatics.mhfire.service.impl;

import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.bizmatics.common.core.bean.CommonPage;
import com.bizmatics.common.core.util.DateUtils;
import com.bizmatics.common.core.util.StringUtils;
import com.bizmatics.mhfire.persistence.mapper.FireStatisticsMapper;

import com.bizmatics.mhfire.persistence.mapper.po.FireStatisticsPO;
import com.bizmatics.mhfire.persistence.mapper.po.UnitPO;
import com.bizmatics.mhfire.service.FireStatisticsService;
import com.bizmatics.mhfire.service.UnitService;

import com.bizmatics.mhfire.service.util.Arith;
import com.bizmatics.mhfire.service.vo.FireBubbleVO;
import com.bizmatics.mhfire.service.vo.FireLevelRatioVO;
import com.bizmatics.mhfire.service.vo.FireUnitPoliceVO;
import net.sf.jsqlparser.statement.alter.Alter;
import org.checkerframework.checker.nullness.Opt;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.text.DecimalFormat;
import java.util.*;
import java.util.stream.Collectors;

/**
 * @author yq
 * @date 2021/5/28 9:35
 */
@Service
public class FireStatisticsServiceImpl implements FireStatisticsService {

    @Autowired
    private FireStatisticsMapper fireStatisticsMapper;
    @Autowired
    private UnitService unitService;
    @Override
    public List<Object> getFireCountByData(Date startTime,Date endTime,String address,String fireType) {
        List<Object> list = new ArrayList<>();
        List<Integer> thisYearList = new ArrayList<>();
        List<Double> monthList = new ArrayList<>();
        List<Double> yearList = new ArrayList<>();
        Date date = new Date();
        //获取传入时间的最大时间和最小时间
        int distanceOfTwoDate =Integer.parseInt(DateUtils.getYear(endTime)) - Integer.parseInt(DateUtils.getYear(startTime)) + 1;
        Date startMax = DateUtils.getFirstDayOfMonth(DateUtils.setMonths(startTime, 0));
        Date endMax = DateUtils.getLastDayOfMonth(DateUtils.setMonths(endTime, 11));
        //获取数据库全部项目的最小时间和最大时间
        Date maxDate = fireStatisticsMapper.selectMaxDate();
        Date minDate = fireStatisticsMapper.selectMinDate();
        int distanceOfTwoDateYear =Integer.parseInt(DateUtils.getYear(maxDate)) - Integer.parseInt(DateUtils.getYear(minDate)) + 1;
        for (int i = 0; i < 12; i++) {
            //获取今年的12个月份数据
            date = DateUtils.setDays(date,1);
            Date setMonths = DateUtils.setMonths(date, i);
            Date firstDayOfMonth = DateUtils.getFirstDayOfMonth(setMonths);
            Date lastDayOfMonth = DateUtils.getLastDayOfMonth(setMonths);
            thisYearList.add(fireStatisticsMapper.selectCount(firstDayOfMonth, lastDayOfMonth, address,fireType,null,null));
            //月平均值
            Integer monthCount = fireStatisticsMapper.selectCountByRangeDate(startMax, endMax, address, fireType,i+1);
            monthList.add(Arith.div(monthCount, distanceOfTwoDate,2));
            Integer yearCount = fireStatisticsMapper.selectCountByRangeDate(null, null,address, fireType,i+1);
            yearList.add(Arith.div(yearCount, distanceOfTwoDateYear,2));
        }
        list.add(thisYearList);
        list.add(monthList);
        list.add(yearList);
        return list;
    }


    @Override
    public CommonPage<FireStatisticsPO> page(Integer current, Integer size, Date startTime, Date endTime, String address) {
        Page<Map<String, String>> page = new Page<>(current, size);
        page = fireStatisticsMapper.page(page, startTime, endTime, address);
        List<FireStatisticsPO> list = new ArrayList<>();
        page.getRecords().forEach(stringObjectMap -> list.add(enhanceFireStatisticsPo(stringObjectMap)));
        return new CommonPage<>(list, page.getTotal(), page.getSize(), page.getCurrent());
    }

    @Override
    public FireStatisticsPO getOne(String id) {
        return enhanceFireStatisticsPo(fireStatisticsMapper.getOne(id));
    }

    @Override
    public FireUnitPoliceVO getFireUnitPolice(String id) {
        //火灾详细信息
        FireStatisticsPO fireStatisticsPo = getOne(id);
        //单位信息
        UnitPO unitPo = new UnitPO();
        unitPo.setAddress(fireStatisticsPo.getAddress());
        unitPo = unitService.getOne(unitPo);

        //TODO 出警记录没有
        FireUnitPoliceVO fireUnitPoliceVO = new FireUnitPoliceVO();
        fireUnitPoliceVO.setFireStatisticsPo(fireStatisticsPo);
        fireUnitPoliceVO.setUnitPo(unitPo);
        return fireUnitPoliceVO;
    }

    @Override
    public List<FireLevelRatioVO> getGroupByLevel(Date startTime, Date endTime, String address) {
        List<FireLevelRatioVO> list = new ArrayList<>();
        int fireCount = 0;
        List<String> fireCauses = fireStatisticsMapper.groupByCause();
        for (String cause:fireCauses) {
            if (StringUtils.isBlank(cause)){
                continue;
            }
            FireLevelRatioVO fireLevelRatioVo = new FireLevelRatioVO();
            fireLevelRatioVo.setFireType(cause);
            Integer count = fireStatisticsMapper.selectCount(startTime, endTime, address, cause,null,null);
            fireLevelRatioVo.setRadio(count.doubleValue());
            list.add(fireLevelRatioVo);
            fireCount+=count;
        }
        //排序
        List<FireLevelRatioVO> orderByList = list.stream().sorted(Comparator.comparing(FireLevelRatioVO::getRadio).reversed()).limit(10).collect(Collectors.toList());

        //差集求和
        double sum = list.stream().filter(item -> !orderByList.contains(item)).mapToDouble(FireLevelRatioVO::getRadio).sum();
        orderByList.stream().filter(item -> "其他".equals(item.getFireType())).findAny()
                .map(fireLevelRatio -> {
                            fireLevelRatio.setRadio(fireLevelRatio.getRadio() + sum);
                            return fireLevelRatio;
                        }
                ).orElseGet(() -> {FireLevelRatioVO fireLevelRatioVo = new FireLevelRatioVO();
                    fireLevelRatioVo.setFireType("其他");
                    fireLevelRatioVo.setRadio(sum);
                    orderByList.add(fireLevelRatioVo);
                    return fireLevelRatioVo;
                });
        int finalFireCount = fireCount;
        if (0 != finalFireCount){
            orderByList.forEach(fireLevelRatio -> fireLevelRatio.setRadio(Arith.div(fireLevelRatio.getRadio(),finalFireCount)));
        }
        return orderByList;
    }


    @Override
    public List<String> getFireType() {
        List<FireLevelRatioVO> list = new ArrayList<>();
        List<String> fireTypes = fireStatisticsMapper.groupByCause().stream().filter(StringUtils::isNotBlank).collect(Collectors.toList());
        for (String cause:fireTypes) {
            FireLevelRatioVO fireLevelRatioVo = new FireLevelRatioVO();
            fireLevelRatioVo.setFireType(cause);
            Integer count = fireStatisticsMapper.selectCount(null, null,null, cause,null,null);
            fireLevelRatioVo.setRadio(count.doubleValue());
            list.add(fireLevelRatioVo);
        }
        return list.stream().sorted(Comparator.comparing(FireLevelRatioVO::getRadio).reversed())
                .map(FireLevelRatioVO::getFireType).collect(Collectors.toList());

    }

    @Override
    public List<String> getAddress() {
        List<FireLevelRatioVO> list = new ArrayList<>();
        List<String> lists = fireStatisticsMapper.getAddress().stream().filter(StringUtils::isNotBlank).filter(s -> !s.contains("派出所")).collect(Collectors.toList());
        for (String address:lists) {
            FireLevelRatioVO fireLevelRatioVo = new FireLevelRatioVO();
            fireLevelRatioVo.setFireType(address);
            Integer count = fireStatisticsMapper.selectCount(null, null,null, null,null,address);
            fireLevelRatioVo.setRadio(count.doubleValue());
            list.add(fireLevelRatioVo);
        }
        return list.stream().sorted(Comparator.comparing(FireLevelRatioVO::getRadio).reversed())
                .map(FireLevelRatioVO::getFireType).collect(Collectors.toList());
    }

    @Override
    public List<FireBubbleVO> getAvgAndSum(Date startTime, Date endTime,String fireType, String unitId) {
        List<FireBubbleVO> list = new ArrayList<>();
        List<String> lists = fireStatisticsMapper.getAddress().stream().filter(StringUtils::isNotBlank).filter(s -> !s.contains("派出所")).collect(Collectors.toList());
        lists.forEach(s -> {
            Map<String, Double> maps = fireStatisticsMapper.selectAvgAndSum(startTime, endTime, s, fireType, unitId);
            FireBubbleVO fireBubbleVo = new FireBubbleVO();
            fireBubbleVo.setRegion(s);
            fireBubbleVo.setFireCount(fireStatisticsMapper.selectCount(startTime,endTime,null,fireType,unitId,s));
            if (null != maps){
                fireBubbleVo.setDeathToll(Optional.ofNullable(maps.get("deathPerson")).map(Double::intValue).orElse(0));
                fireBubbleVo.setFireArea(Arith.div(maps.get("fireArea"),1));
            }else {
                fireBubbleVo.setDeathToll(0);
                fireBubbleVo.setFireArea(0.00);
            }
            list.add(fireBubbleVo);
        });
        return list;
    }

    public FireStatisticsPO enhanceFireStatisticsPo(Map<String,String> map){FireStatisticsPO fireStatisticsPo = new FireStatisticsPO();
        fireStatisticsPo.setId(Optional.ofNullable(map.get("火灾序号")).filter(s -> !"null".equals(s)).orElse(""));
        fireStatisticsPo.setAddress(Optional.ofNullable(map.get("起火地点")).filter(s -> !"null".equals(s)).orElse(""));
        fireStatisticsPo.setBurnedArea(Optional.ofNullable(map.get("过火面积")).filter(s -> !"null".equals(s)).orElse(""));
        fireStatisticsPo.setPropertyLoss(Optional.ofNullable(map.get("直接财产损失")).filter(s -> !"null".equals(s)).orElse(""));
        fireStatisticsPo.setDeathToll(Optional.ofNullable(map.get("死亡人数")).filter(s -> !"null".equals(s)).orElse(""));
        fireStatisticsPo.setNonFatal(Optional.ofNullable(map.get("受灾人数")).filter(s -> !"null".equals(s)).orElse(""));
        fireStatisticsPo.setDisasterHome(Optional.ofNullable(map.get("受灾户数")).filter(s -> !"null".equals(s)).orElse(""));
        fireStatisticsPo.setFireCause(Optional.ofNullable(map.get("火灾原因分类（一级）")).filter(s -> !"null".equals(s)).orElse(""));
        fireStatisticsPo.setFireLevel(Optional.ofNullable(map.get("火灾等级")).filter(s -> !"null".equals(s)).orElse(""));
        return fireStatisticsPo;
    }
}
